<?php

use Tree\Tree;
use Tree\Node;

class Megamenu extends Model{

    const TYPE_TEXTE = 1;

    protected $table = 'megamenu';

    /** @var Tree */
    protected static $tree;

    public function get($formvars = array(), $champs = "")
    {
        $lgId = Boutique::getInstance()->getLangue()->id;

        $sql ="SELECT m.megamenu_id";
        $sql.=" ,m.megamenu_ouverture";
        $sql.=" ,m.megamenu_colone";
        $sql.=" ,m.megamenu_parent_id";
        $sql.=" ,m.megamenu_ext_id";
        $sql.=" ,m.megamenu_rang";
        $sql.=" ,mn.megamenu_nom_id";
        $sql.=" ,mn.megamenu_nom_nom";
        $sql.=" ,mn.megamenu_nom_lien";

        if (!empty($formvars['niveau']) || !empty($formvars['niveau_only'])) {
            $sql .= " ,LENGTH(m.megamenu_ext_id) - LENGTH(REPLACE(m.megamenu_ext_id, ',', '')) + 1 AS niveau";
        }

        $sql.=" FROM megamenu m";
        $sql.=" LEFT JOIN megamenu_nom mn ON (mn.megamenu_id = m.megamenu_id AND mn.langue_id = '{$lgId}')";
        if(empty($formvars['no_boutique'])) {
            $sql .= " INNER JOIN megamenu_boutique mb ON (m.megamenu_id = mb.megamenu_id AND mb.boutique_id = '" . Boutique::getInstance()->id . "')";
        }
        $sql.=" WHERE 1";

        if (!empty($formvars['megamenu_id'])) {
            if (is_array($formvars['megamenu_id'])) {
                $sql .= " AND m.megamenu_id IN (" . implode(',', $formvars['megamenu_id']) . ")";
            } else {
                $sql .= " AND m.megamenu_id = '" . $formvars['megamenu_id'] . "'";
            }
        }

        if(empty($formvars['no_boutique'])){
            $sql .= " AND mb.boutique_id = " . Boutique::getInstance()->id;
        }

        $sql .= " AND mb.megamenu_boutique_statut_id = 1";
        $sql .= " AND m.megamenu_statut_id = 1";

        if (!empty($formvars['megamenu_id_not_in'])) {
            $sql .= " AND m.megamenu_id NOT IN (" . $formvars['megamenu_id_not_in'] . ")";
        }
        if (!empty($formvars['megamenu_parent_id'])) {
            $sql .= " AND m.megamenu_parent_id = '" . $formvars['megamenu_parent_id'] . "'";
        }
        if (!empty($formvars['only_parent'])) {
            $sql .= " AND m.megamenu_parent_id IS NULL";
        }
        if (!empty($formvars['no_parent'])) {
            $sql .= " AND m.megamenu_parent_id IS NOT NULL";
        }
        if (!empty($formvars['megamenu_ext_id'])) {
            $sql .= " AND FIND_IN_SET(".$formvars['megamenu_ext_id'].",m.megamenu_ext_id) > 0";
        }

        $sql .= " GROUP BY m.megamenu_id";

        $sql .= " HAVING 1";

        if(!empty($formvars["order"])){
            switch ($formvars["order"])
            {
                case "id-rang":
                    $sql.= " ORDER BY m.megamenu_parent_id ASC";
                    $sql.= " , m.megamenu_rang ASC";
                    break;
                default:
                    $sql.= " ORDER BY m.megamenu_id";
                    break;
            }
        }

        if (empty($champs)) {
            $this->sql->query($sql, SiteSQL::SQL_ALL, SiteSQL::SQL_ASSOC);
            return $this->sql->record;
        } else {
            $this->sql->query($sql, SiteSQL::SQL_INIT, SiteSQL::SQL_ASSOC);

            return $this->sql->record[$champs];
        }
    }

    /**
     * @param $megamenu_id
     */
    public function updateExtId($megamenu_id)
    {

        $megamenu_ext_id = $megamenu_id;
        $arrMegamenus[0]['megamenu_parent_id'] = $megamenu_id;
        do {
            $arrMegamenus = $this->get(array(
                'megamenu_id' => $arrMegamenus[0]['megamenu_parent_id'],
            ));
            if (!empty($arrMegamenus[0]['megamenu_parent_id'])) {
                $megamenu_ext_id .= "," . $arrMegamenus[0]['megamenu_parent_id'];
            }
        } while (!empty($arrMegamenus[0]['megamenu_parent_id']));

        $sql = " UPDATE megamenu SET megamenu_ext_id = '" . $megamenu_ext_id . "'";
        $sql .= " WHERE megamenu_id = '" . $megamenu_id . "'";
        $this->sql->query($sql);

        // on met a jour les enfants
        $sql = "SELECT megamenu_id FROM megamenu WHERE megamenu_parent_id = '" . $megamenu_id . "'";
        $this->sql->query($sql, SiteSQL::SQL_ALL, SiteSQL::SQL_ASSOC);
        $arrEnfant = $this->sql->record;
        if (!empty($arrEnfant)) {
            foreach ($arrEnfant as $enfant) {
                $this->updateExtId($enfant['megamenu_id']);
            }
        }
    }

    /**
     * Retourne un arbre avec toutes les megamenus du site
     *
     * @param array $formvars
     * @return bool|Node|Tree
     * @throws Exception
     */
    public static function getTree($formvars = [])
    {
        if (is_null(self::$tree)) {
            self::$tree = new Tree();

            $megamenu = new Megamenu();
            $megamenus = $megamenu->get(["order" => "id-rang"]);

            foreach ($megamenus as $mega) {
                if (empty($mega['megamenu_parent_id'])) {
                    self::$tree->addNode(new Node($mega));
                } else {
                    $parent = self::$tree->findNode(['megamenu_id' => $mega['megamenu_parent_id']]);
                    if ($parent instanceof Node) {
                        $parent->addChildren(new Node($mega));
                    }
                }
            }
        }
        if (!empty($formvars)) {
            return self::$tree->findNode($formvars);
        } else {
            return self::$tree;
        }
    }
}
