<?php

/**
 * https://preprod-admin.paybox.com
 * 199988832 / 1999888I
 *
 * configuration paybox
 * $config['paybox']['site'] = '1999888';
 * $config['paybox']['rang'] = '32';
 * $config['paybox']['identifiant'] = '1686319';
 * $config['paybox']['key'] = '0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF';
 * $config['paybox']['mode'] = 'dev'|'prod'
 */

namespace Shopbydedi\Payment;

use Panier;
use Tpe;
use Boutique;
use Application;

/**
 * Class Paybox
 * @package Shopbydedi\Payment
 */
abstract class Paybox implements PaymentInterface
{
    protected $site;
    protected $rang;
    protected $identifiant;
    protected $key;
    protected $server = 'tpeweb.paybox.com';
    /**
     * @var \Router
     */
    protected $router;

    /**
     * @param $config
     *
     * @throws \DediException
     */
    public function __construct($config)
    {

        if (empty($config['paybox'])) {
            throw new \DediException('configure infos paybox in boutique config');
        }

        $this->site = $config['paybox']['site'];
        $this->rang = $config['paybox']['rang'];
        $this->identifiant = $config['paybox']['identifiant'];
        $this->key = $config['paybox']['key'];
        if ($config['paybox']['mode'] == 'dev') {
            $this->server  = 'preprod-' . $this->server;
        }

        $this->router = Application::getRouter();
    }

    /**
     * @param Tpe $tpe
     * @param $amount
     *
     * @return array
     */
    public function bootRequest(Tpe $tpe, $amount)
    {
        $params = array();
        $params['PBX_SITE'] = $this->site;
        $params['PBX_RANG'] = $this->rang;
        $params['PBX_IDENTIFIANT'] = $this->identifiant;
        $params['PBX_TOTAL'] = round($amount * 100);
        $params['PBX_DEVISE'] = '978';
        $params['PBX_CMD'] = $tpe->tpe_reference_id;
        $params['PBX_PORTEUR'] = $tpe->getclient()->client_email;

        $params['PBX_RETOUR'] = 'Mt:M;Ref:R;Auto:A;Erreur:E';
        $params['PBX_HASH'] = 'SHA512';
        $params['PBX_TIME'] = date('c');

        $params['PBX_REPONDRE_A'] = substr(SITE_URL, 0, strlen(SITE_URL) - 1) . $this->router->getRouter()->generate("traitement_actif", array('class' => 'paybox'));

        // retour automatique
        $params['PBX_WAIT'] = 0;
        $params['PBX_TXT'] = '';
        $params['PBX_BOUTPI'] = 'nul';
        $params['PBX_BKGD'] = '#FFFFFF';
        // retour automatique

        return $params;
    }

    /**
     * @param Tpe $tpe
     * @param float $amount
     * @param array $params
     *
     * @return string
     */
    public function request(Tpe $tpe, $amount, $params = array())
    {

        $params = array_merge($this->bootRequest($tpe, $amount), $params);
        
        $params['PBX_EFFECTUE'] = substr(SITE_URL, 0, strlen(SITE_URL) - 1) . $this->router->getRouter()->generate('paiement_success', ['_locale' => Boutique::getInstance()->getLangue()->ini]);

        $retourError = Application::getCurrentRequest()->getRequestUri();
        $params['PBX_REFUSE']  = substr(SITE_URL, 0, strlen(SITE_URL) - 1) . $retourError;
        $params['PBX_ANNULE'] = substr(SITE_URL, 0, strlen(SITE_URL) - 1) . $retourError;
        //page en cas d'erreur
        $params['PBX_ERREUR'] = substr(SITE_URL, 0, strlen(SITE_URL) - 1) . $retourError;

        /* non actif
        // deactivation du 3ds
        if($amount < 100) {
            $params['PBX_3DS'] = '1';
        }
        */

        return $this->generateFormRequest($params);
    }

    /**
     * @param Panier $panier
     * @param $amount
     * @param $params
     * @return string
     */
    public function requestFromPanier(Panier $panier, $amount, $params = array())
    {
        $params = array_merge($this->bootRequest(new Tpe($panier->getReference()), $amount), $params);

        $deviceType = (Application::getMobileDetect()->isMobile() && !Application::getMobileDetect()->isTablet()) ?
            'mobile' : 'desktop';

        $retourError = ($deviceType == 'desktop') ? Boutique::getInstance()->getNextPageProcess() : 'recapitulatif';

        $params['PBX_EFFECTUE'] = substr(SITE_URL, 0, strlen(SITE_URL) - 1) . $this->router->getRouter()->generate('commande', ['_locale' => Boutique::getInstance()->getLangue()->ini]);
        $params['PBX_REFUSE']  = substr(SITE_URL, 0, strlen(SITE_URL) - 1) . $this->router->getRouter()->generate($retourError);
        $params['PBX_ANNULE'] = substr(SITE_URL, 0, strlen(SITE_URL) - 1) . $this->router->getRouter()->generate($retourError);
        //page en cas d'erreur
        $params['PBX_ERREUR'] = substr(SITE_URL, 0, strlen(SITE_URL) - 1) . $this->router->getRouter()->generate($retourError);

        return $this->generateFormRequest($params);
    }

    /**
     * @param $params
     *
     * @return string
     */
    protected function generateFormRequest($params)
    {
        $msg = '';
        foreach ($params as $k => $v) {
            if (!empty($msg)) {
                $msg .= '&';
            }
            $msg .= $k . '=' . $v;
        }
        $binKey = pack("H*", $this->key);
        $params['PBX_HMAC'] = strtoupper(hash_hmac('sha512', $msg, $binKey));

        $out = '';
        $out .= '<form method="POST" action="https://' . $this->server . '/cgi/MYchoix_pagepaiement.cgi">';
        foreach ($params as $k => $v) {
            $out .= '<input type="hidden" name="' . $k . '" value="' . $v . '" />';
        }
        $out .= '<input type="image" src="/styles/images/paiement/paiement_paybox.jpg" onclick="this.form.submit();"/>';
        $out .= '</form>';
        return $out;
    }


    /**
     * @param array $request
     * @throws \DediException
     *
     * @return void
     */
    function response($request)
    {

        if (!empty($request['Ref'])) {

            /**
             * L'execution est ok on traite les cas
             * 00 Transaction approuvee ou traitee avec succes
             */

            // XXXXXX : pour les tests

            $info = '';
            foreach ($request as $k => $v) {
                $info .= $k . ' = ' . $v . "<br/>";
            }

            if ($request['Erreur'] == "00000" || $request['Erreur'] == "XXXXXX") {

                $tpe = new Tpe;
                $tpe->paiementOK($request['Ref'], $request);

            } else {

                $tpe = new Tpe;
                $tpe->paiementKO($request['Ref'], $request);
                unset($_POST);
            }
        }
    }
}
